import streamlit as st
import pandas as pd
import numpy as np

from sklearn.preprocessing import LabelEncoder

import viewutil

LABEL_COLUMNS = 'Columns'
LABEL_DATE_COLUMNS = 'Date Columns'
LABEL_CATEGORICAL_COLUMNS = 'Categorical Columns'
LABEL_NUMERICAL_COLUMNS = 'Numerical Columns'
LABEL_OPERATIONS = 'Operations'
LABEL_DROP_NULL_VALUES = 'Drop Null Values'
LABEL_DROP_DUPLICATES = 'Drop Duplicates'
LABEL_RESET_INDEX = 'Reset Index'
LABEL_ENCODER = 'Encoder'
LABEL_ENCODER_TYPE = 'Encoder Type'
LABEL_ENCODER_COLUMNS = 'Encode Columns'

ENCODER_TYPE_ONE_HOT = 'One Hot Encoder'
ENCODER_TYPE_LABEL = 'Label Encoder'
ENCODERS_TYPES = [
    ENCODER_TYPE_ONE_HOT,
    ENCODER_TYPE_LABEL
]

def generate_date_process_view(df):
    columns = st.multiselect(LABEL_COLUMNS, list(df.columns), list(df.columns))
    df = df[columns]
    
    cat_col = df.select_dtypes(include=['object']).columns.tolist()
    num_col = df.select_dtypes(include=np.number).columns.tolist()
    
    date_columns = st.multiselect(LABEL_DATE_COLUMNS, cat_col)
    for col in date_columns:
        try:
            df[col] = pd.to_datetime(df[col])
        except Exception as e:
            st.error(f'Column \'{col}\' cannot be converted to date/time')

    if cat_col is not None and date_columns is not None:
        cat_col = list(set(cat_col) - set(date_columns))

    categorical_columns = st.multiselect(LABEL_CATEGORICAL_COLUMNS, columns, cat_col)
    nummeric_columns = st.multiselect(LABEL_NUMERICAL_COLUMNS, num_col, num_col)

    viewutil.section_title(LABEL_OPERATIONS)

    if st.checkbox(LABEL_DROP_NULL_VALUES):
        df.dropna(inplace=True)

    if st.checkbox(LABEL_DROP_DUPLICATES):
        df.drop_duplicates(inplace=True)

    if st.checkbox(LABEL_RESET_INDEX):
        df.reset_index(drop=True, inplace=True)

    if st.checkbox(LABEL_ENCODER):
        encoder_type = st.selectbox(LABEL_ENCODER_TYPE, ENCODERS_TYPES)
        encode_columns = st.multiselect(LABEL_ENCODER_COLUMNS, categorical_columns)

        if len(encode_columns):
            if encoder_type == ENCODER_TYPE_LABEL:
                df.loc[:,encode_columns] = df.loc[:,encode_columns].apply(LabelEncoder().fit_transform)
            elif encoder_type == ENCODER_TYPE_ONE_HOT:
                df = pd.get_dummies(df, columns=encode_columns, prefix=encode_columns )


    viewutil.view_data(df, "process")

    return df, categorical_columns, nummeric_columns


import streamlit as st

def section_title(text):
    st.markdown(f'*{text}*')

def view_data(df, checkbox_key=None):
    if df is not None:
        section_title('Shape')
        st.write(df.shape)

        section_title('Head')
        st.write(df[:5])

        if st.checkbox("View All", key=checkbox_key):
            st.write(df)

def st_plot(plot):
    try:
        st.pyplot(plot.get_figure(), clear_figure=True)
    except Exception as e:
        st.error(e)

def st_round(x, precision=3):
    st.write(round(x, precision))


import pydeck as pdk

MAPBOX_STYLES = ['light-v10', 'dark-v10', 'streets-v11', 'satellite-v9', 'satellite-streets-v11']

def get_mapbox_styles():
    return MAPBOX_STYLES

def get_mapbox_map(data, lat_col, lon_col, lat, lon, zoom=11, pitch=50, radius=100, style=MAPBOX_STYLES[0]):
    return pdk.Deck(
        map_style="mapbox://styles/mapbox/" + style,
        initial_view_state={
            "latitude": lat,
            "longitude": lon,
            "zoom": zoom,
            "pitch": pitch,
        },
        layers=[
            pdk.Layer(
                "HexagonLayer",
                data=data,
                get_position=[lon_col, lat_col],
                radius=radius,
                elevation_scale=4 if pitch > 0 else 0,
                elevation_range=[0, 1000],
                pickable=True,
                extruded=True,
            ),
        ]
    )


import streamlit as st
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LinearRegression, LogisticRegression
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier
from sklearn.metrics import confusion_matrix, r2_score

import viewutil

import matplotlib.pyplot as plt

LABEL_FEATURE_COLUMNS = 'Feature Columns'
LABEL_PREDICTION_COLUMN = 'Label Column'
LABEL_LEARNING_TYPE = 'Learning Type'
LABEL_ALGORITHM = 'Algorithm'
LABEL_TRAIN_SIZE = 'Train Size'
LABEL_TRAIN_MODEL = 'Train Model'
LABEL_EVALUATION_METRICS = 'Evaluate'
LABEL_R2_SCORE = 'R^2 Score'
LABEL_CONFUSION_MATRIX = 'Confusion Matrix'
LABEL_COEFFICIENTS = 'Coefficients'
LABEL_INTERCEPT = 'Intercept'

ML_TYPE_CLASSIFICATION = 'Classification'
ML_TYPE_REGRESSION = 'Regression'

ALGO_DECISION_TREE_CLASSIFIER = 'Decision Tree'
ALGO_LOGISTIC_REGRESSION = 'Logistic Regression'
ALGO_SVM = 'SVM'

ALGO_LINEAR_REGRESSION = 'Linear Regression'

ML_TYPES = {
    ML_TYPE_CLASSIFICATION: [
        ALGO_DECISION_TREE_CLASSIFIER,
        ALGO_LOGISTIC_REGRESSION,
        ALGO_SVM
    ],
    ML_TYPE_REGRESSION: [
        ALGO_LINEAR_REGRESSION
    ]
}

def generate_train_view(df, random_state):
    columns = list(df.columns)
    
    col1, col2 = st.beta_columns(2)
    with col1:
        ml_type = st.selectbox(LABEL_LEARNING_TYPE, list(ML_TYPES.keys()))
    with col2:
        algo_type = st.selectbox(LABEL_ALGORITHM, ML_TYPES.get(ml_type))

    feature_columns = st.multiselect(LABEL_FEATURE_COLUMNS, columns, columns[:-1])
    prediction_column = st.selectbox(LABEL_PREDICTION_COLUMN, list(set(columns) - set(feature_columns)), 0)
    train_size = st.slider(LABEL_TRAIN_SIZE, min_value=5, max_value=95, value=70, step=5)

    if len(feature_columns) and len(prediction_column) and st.checkbox(LABEL_TRAIN_MODEL):
        model = None
        X_train, X_test, y_train, y_test = train_test_split(df[feature_columns], df[prediction_column], train_size = train_size / 100, random_state = random_state)
        if algo_type == ALGO_LINEAR_REGRESSION:
            model = LinearRegression()
        elif algo_type == ALGO_LOGISTIC_REGRESSION:
            model = LogisticRegression(solver='lbfgs', multi_class='auto', random_state=random_state)
        elif algo_type == ALGO_SVM:
            model = SVC(kernel='linear', random_state=random_state)
        elif algo_type == ALGO_DECISION_TREE_CLASSIFIER:
            model = DecisionTreeClassifier(criterion='entropy')

        if model is not None:
            try:
                model.fit(X_train, y_train)
                y_pred = model.predict(X_test)
            
                score = model.score(X_test, y_test)
                st.info('Accuracy: **' + str(round(score * 100, 2)) + '%**')
                
                if st.checkbox(LABEL_EVALUATION_METRICS):
                    if ml_type == ML_TYPE_REGRESSION:
                        viewutil.section_title(LABEL_R2_SCORE)
                        viewutil.st_round(r2_score(y_test, y_pred))

                        viewutil.section_title(LABEL_INTERCEPT)
                        viewutil.st_round(model.intercept_)

                        viewutil.section_title(LABEL_COEFFICIENTS)
                        st.write(model.coef_)

                        if X_test.shape[1] == 1 and len(X_test) == len(y_test):
                            plt.scatter(X_test, y_test, color="red")
                            plt.plot(X_test, y_pred, color="green")
                            plt.title("Test Data")
                            st.pyplot(plt.figure(1))

                    if ml_type == ML_TYPE_CLASSIFICATION:
                        viewutil.section_title(LABEL_CONFUSION_MATRIX)
                        st.write(pd.DataFrame(confusion_matrix(y_test, y_pred)))                    

            except Exception as e:
                st.error(e)



import streamlit as st
import matplotlib.pyplot as plt
import seaborn as sns

PLOT_TYPE_BOX = 'Box'
PLOT_TYPE_COUNT = 'Count'
PLOT_TYPE_DIST = 'Distribution'
PLOT_TYPE_HISTOGRAM = 'Histogram'
PLOT_TYPE_LINE = 'Line'
PLOT_TYPE_SCATTER = 'Scatter'
PLOT_TYPE_SWARM = 'Swarm'
PLOT_TYPES = [
    PLOT_TYPE_BOX,
    PLOT_TYPE_COUNT,
    PLOT_TYPE_DIST,
    PLOT_TYPE_HISTOGRAM,
    PLOT_TYPE_LINE,
    PLOT_TYPE_SCATTER,
    PLOT_TYPE_SWARM
]

LABEL_PLOT_TYPE = 'Plot Type'
LABEL_X = 'X'
LABEL_Y = 'Y'
LABEL_Z = 'Hue'

def st_plot(plot):
    try:
        st.pyplot(plot.get_figure(), clear_figure=True)
    except Exception as e:
        st.error(e)

def generate_plot_view(df):
    x = y = z = None
    plot_type = st.selectbox(LABEL_PLOT_TYPE, PLOT_TYPES)
    col1, col2, col3 = st.beta_columns(3)
    with col1:
        x = st.selectbox(LABEL_X, [None] + list(df.columns))
    with col2:
        if plot_type not in [PLOT_TYPE_COUNT, PLOT_TYPE_DIST, PLOT_TYPE_HISTOGRAM]:
            y = st.selectbox(LABEL_Y, [None] + list(df.columns))
    with col3:
        if plot_type not in [PLOT_TYPE_COUNT, PLOT_TYPE_DIST, PLOT_TYPE_HISTOGRAM]:
            z = st.selectbox(LABEL_Z, [None] + list(df.columns))

    try:
        if plot_type == PLOT_TYPE_BOX:
            st_plot(sns.boxplot(x=x, y=y, hue=z, data=df))
        elif plot_type == PLOT_TYPE_COUNT:
            st_plot(sns.countplot(x=x, data=df))
        elif plot_type == PLOT_TYPE_DIST:
            st_plot(sns.distplot(df[x]))        
        elif plot_type == PLOT_TYPE_HISTOGRAM:
            st_plot(sns.histplot(df[x]))
        elif plot_type == PLOT_TYPE_LINE:
            st_plot(sns.lineplot(x=x, y=y, hue=z, data=df))
        elif plot_type == PLOT_TYPE_SCATTER:
            st_plot(sns.scatterplot(x=x, y=y, hue=z, data=df))
        elif plot_type == PLOT_TYPE_SWARM:
            st_plot(sns.swarmplot(x=x, y=y, hue=z, data=df))
    except Exception as e:
        if str(e) != "None":
            st.error(e)


import streamlit as st
import numpy as np
import maputil

LABEL_LATITUDE_COLUMN = 'Latitude Column'
LABEL_LONGITUDE_COLUMN = 'Longitude Column'
LABEL_ZOOM = 'Zoom'
LABEL_PITCH = 'Pitch'
LABEL_STYLE = 'Style'
LABEL_RADIUS = 'Radius'

import commonutil

def generate_map_view(df):
    lat_col = lon_col = None
    columns = list(df.columns)
    col1, col2, col3 = st.beta_columns(3)
    with col1:
        lat_col = st.selectbox(LABEL_LATITUDE_COLUMN, [''] + columns, commonutil.find_index_from_list(columns, 'lat') + 1)
        zoom = st.number_input(LABEL_ZOOM, 1, 20, 11)
    with col2:
        lon_col = st.selectbox(LABEL_LONGITUDE_COLUMN, [''] + columns, commonutil.find_index_from_list(columns, 'lon') + 1)
        pitch = st.number_input(LABEL_PITCH, 0, 100, 50)
    with col3:
        map_style = st.selectbox(LABEL_STYLE, maputil.get_mapbox_styles())
        radius = st.number_input(LABEL_RADIUS, 1, 1000000, 100)

    
    if lat_col is not '' and lon_col is not '':
        midpoint = (np.average(df[lat_col]), np.average(df[lon_col]))
        st.write(maputil.get_mapbox_map(df, lat_col, lon_col, midpoint[0], midpoint[1], zoom=zoom, pitch=pitch, radius=radius, style=map_style))


from numpy.lib.utils import info
import streamlit as st
import pandas as pd
import numpy as np
import io
import matplotlib.pyplot as plt
import seaborn as sns

import sys
sys.path = list(set(['eda', 'data', 'ml', 'utils'] + sys.path))

import dataprocessview
import edamapview
import edacustomplot
import mlsupervised
import viewutil
import commonutil

APP_TITLE = "ActiveML"
APP_ICON = "🔮"

SIDEBAR_GROUP_EDA = "EDA"
SIDEBAR_GROUP_ML = "ML"
SIDEBAR_GROUP_SETTINGS = "Settings"

DATA_VIEW_PROCESS = 'Process Data'

EDA_VIEW_BASIC = 'Basic Analysis'
EDA_VIEW_CATEGORICAL = 'Categorical Analysis'
EDA_VIEW_NUMERICAL = 'Numerical Analysis'
EDA_VIEW_BIVARIATE = 'Bivariate Analysis'
EDA_VIEW_MULTIVARIATE = 'Multivariate Analysis'
EDA_VIEW_MAP = 'Map View'
EDA_VIEW_CUSTOM_PLOT = 'Custom Plot'

EDA_VIEWS = [
    EDA_VIEW_BASIC,
    EDA_VIEW_CATEGORICAL,
    EDA_VIEW_NUMERICAL,
    EDA_VIEW_BIVARIATE,
    EDA_VIEW_MULTIVARIATE,
    EDA_VIEW_MAP,
    EDA_VIEW_CUSTOM_PLOT
]

ML_VIEW_SUPERVISED = 'Supervised'

ML_VIEWS = [
    ML_VIEW_SUPERVISED
]

DATA_SOURCE_FILE = "Local"
DATA_SOURCE_URL = "URL"
DATA_SOURCE_DEMO = "Demo"
DATA_SOURCES = [
    DATA_SOURCE_FILE,
    DATA_SOURCE_URL,
    DATA_SOURCE_DEMO
]

DATA_MAX_N_ROWS = 100000
DATA_CSV_SEPERATORS = [',', ';', '', '|', r'\t']

DEMO_DATASETS = {
    '': {
        'url': ''
    },
    'Iris': {
        'url': 'https://gist.githubusercontent.com/curran/a08a1080b88344b0c8a7/raw/0e7a9b0a5d22642a06d3d5b9bcbad9890c8ee534/iris.csv',
        'nrows': 150,
        'sep': ','
    },
    'Uber': {
        'url': 'https://s3-us-west-2.amazonaws.com/streamlit-demo-data/uber-raw-data-sep14.csv.gz',
        'nrows': 10000,
        'sep': ','
    },
    'US Sate Population': {
        'url': 'https://raw.githubusercontent.com/jakevdp/data-USstates/master/state-population.csv',
        'nrows': 3000,
        'sep': ','
    }
}



@st.cache
def load_data(file, nrows=None, sep=','):
    data = pd.read_csv(file, nrows=nrows, sep=sep)
    lowercase = lambda x: str(x).lower()
    data.rename(lowercase, axis='columns', inplace=True)
    return data



def main():
    df = None
    selected_eda_views = []
    selected_ml_views = []
    categorical_columns = None
    nummeric_columns = None
    random_state = None

    st.beta_set_page_config(
        page_title=APP_TITLE,
        page_icon=APP_ICON,
        layout="centered",
        initial_sidebar_state="auto")

    st.title(APP_ICON + ' ' + APP_TITLE)

    with st.beta_expander("Load Data", True):
        dataset_source = st.selectbox('Data Source', DATA_SOURCES)
        
        file = None
        nrows = DATA_MAX_N_ROWS
        sep = ','

        if dataset_source == DATA_SOURCE_DEMO:
            dataset_name = st.selectbox('Demo Dataset', list(DEMO_DATASETS.keys()))
            if len(dataset_name):
                file = DEMO_DATASETS[dataset_name]['url']
                nrows = DEMO_DATASETS[dataset_name]['nrows']
                sep = DEMO_DATASETS[dataset_name]['sep']

        if dataset_source != DATA_SOURCE_DEMO:
            col1, col2 = st.beta_columns(2)
            with col1:
                nrows = st.number_input('Number of Rows', value=nrows)
            with col2:
                sep = st.selectbox('Sepetator', DATA_CSV_SEPERATORS)
                sep = sep if len(sep) else ','
            nrows = nrows if nrows else None

        if dataset_source == DATA_SOURCE_FILE:
            file = st.file_uploader('Upload File', type=['csv', 'tsv', 'txt'])
            if file is not None:
                file.seek(0)

        if dataset_source == DATA_SOURCE_URL:
            file = st.text_input('Dataset URL (csv)')
            if len(file) == 0:
                file = None

        if file is not None:
            try:
                df = load_data(file, nrows, sep)
                viewutil.view_data(df, 'file')
            except Exception as e:
                st.error(e)
                st.stop()
    
    if df is not None and len(df):
        
        with st.beta_expander(DATA_VIEW_PROCESS):
            df, categorical_columns, nummeric_columns = dataprocessview.generate_date_process_view(df)

        if len(categorical_columns) == 0:
            EDA_VIEWS.remove(EDA_VIEW_CATEGORICAL)

        if len(nummeric_columns) == 0:
            EDA_VIEWS.remove(EDA_VIEW_NUMERICAL)
        
        if len(categorical_columns) == 0 or len(nummeric_columns) == 0:
            EDA_VIEWS.remove(EDA_VIEW_BIVARIATE)
            EDA_VIEWS.remove(EDA_VIEW_MULTIVARIATE)

        if commonutil.find_index_from_list(df.columns, 'lat') == -1 or commonutil.find_index_from_list(df.columns, 'lat') == -1:
            EDA_VIEWS.remove(EDA_VIEW_MAP)

        with st.sidebar.beta_expander(SIDEBAR_GROUP_EDA):
            for view in EDA_VIEWS:
                if st.checkbox(view):
                    selected_eda_views.append(view)

        with st.sidebar.beta_expander(SIDEBAR_GROUP_ML, False):
            for view in ML_VIEWS:
                if st.checkbox(view):
                    selected_ml_views.append(view)

        with st.sidebar.beta_expander(SIDEBAR_GROUP_SETTINGS, False):
            if st.checkbox("Random State", True):
                random_state = st.number_input("Random Seed", 42)

        if len(selected_eda_views):
            st.markdown('## Exploratory Data Analysis')

        if EDA_VIEW_BASIC in selected_eda_views:
            with st.beta_expander(EDA_VIEW_BASIC, True):
                viewutil.section_title('Info')
                buffer = io.StringIO()
                df.info(buf=buffer)
                st.text(buffer.getvalue())

                viewutil.section_title('Data Types')
                dtypes = df.dtypes.rename('Total')
                dtypes

                if len(df.columns):
                    viewutil.section_title('Describe')
                    desc = df.describe().T
                    desc

                viewutil.section_title('Missing Values')
                isnull = df.isnull().sum().rename('Total')
                isnull

                viewutil.section_title('Skew')
                skew = df.skew()
                skew
                
                viewutil.section_title('Covariance')
                cov = df.cov()
                cov
                if len(cov) and st.checkbox('Hitmap', key='cov_hitmap'):
                    viewutil.st_plot(sns.heatmap(cov, annot=True))

                viewutil.section_title('Correlation')
                corr = df.corr()
                corr
                if len(corr) and st.checkbox('Hitmap', key='corr_hitmap'):
                    viewutil.st_plot(sns.heatmap(corr, annot=True))
        
        if EDA_VIEW_CATEGORICAL in selected_eda_views:
            with st.beta_expander(EDA_VIEW_CATEGORICAL, True):
                selected_columns = st.multiselect('Columns', categorical_columns, key='cat_col')
                for col in selected_columns:
                    viewutil.section_title(col)
                    col1, col2 = st.beta_columns(2)
                    with col1:
                        count = df[col].value_counts()
                        count

                    with col2:
                        viewutil.st_plot(sns.countplot(df[col]))

        if EDA_VIEW_NUMERICAL in selected_eda_views:
            with st.beta_expander(EDA_VIEW_NUMERICAL, True):
                selected_columns = st.multiselect('Columns', nummeric_columns, key='num_col')
                for col in selected_columns:
                    viewutil.section_title(col)
                    col1, col2 = st.beta_columns(2)
                    with col1:
                        desc = df[col].describe()
                        desc
                    with col2:
                        try:
                            viewutil.st_plot(sns.distplot(df[col]))
                        except Exception as e:
                            st.error(e)

        if EDA_VIEW_BIVARIATE in selected_eda_views:
            with st.beta_expander(EDA_VIEW_BIVARIATE, True):
                selected_cat_columns = st.multiselect('X', categorical_columns, key='bi_cat_col')
                selected_num_columns = st.multiselect('Y', nummeric_columns, key='bi_num_col')
                for cat_col in selected_cat_columns:
                    for num_col in selected_num_columns:
                        if cat_col != num_col:
                            col1, col2 = st.beta_columns(2)
                            with col1:
                                desc = df[num_col].describe()
                                desc
                            
                            with col2:
                                viewutil.st_plot(sns.swarmplot(x=df[cat_col], y=df[num_col]))

                for cat_col in selected_cat_columns:
                    for num_col in selected_num_columns:
                        if cat_col != num_col:
                            viewutil.st_plot(sns.boxplot(x=df[cat_col], y=df[num_col]))

        if EDA_VIEW_MULTIVARIATE in selected_eda_views:
            with st.beta_expander(EDA_VIEW_MULTIVARIATE, True):
                selected_x_columns = st.multiselect('X', nummeric_columns, key='mul_num_col')
                selected_y_columns = st.multiselect('Y', nummeric_columns, key='mul_num_col')
                selected_hue_columns = st.multiselect('Hue', categorical_columns, key='mul_cat_col')
                
                for x in selected_x_columns:
                    for y in selected_y_columns:
                        if x != y:
                            for z in selected_hue_columns:
                                viewutil.st_plot(sns.scatterplot(df[x], df[y], df[z]))

        if EDA_VIEW_MAP in selected_eda_views:
            with st.beta_expander(EDA_VIEW_MAP, True):
                edamapview.generate_map_view(df)

        if EDA_VIEW_CUSTOM_PLOT in selected_eda_views:
            with st.beta_expander(EDA_VIEW_CUSTOM_PLOT, True):
                edacustomplot.generate_plot_view(df)

        if len(selected_ml_views):
            st.markdown('## Machine Learning')

        if ML_VIEW_SUPERVISED in selected_ml_views:
            with st.beta_expander(ML_VIEW_SUPERVISED, True):
                mlsupervised.generate_train_view(df, random_state)

main()